/* Copyright (c) 2015, Electric Cloud, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* (As an historical note, the above license instantiates
 * the "BSD 2-Clause License" template that was presented
 * by the Open Source Initiative at
 *
 *   http://www.opensource.org/licenses/bsd-license.php
 *
 * on 2012-02-23.)
 */

/* Summary:
 *
 * Declares an implementation of vrpsx_allocator_t that
 * tracks how it is used for the purpose of testing code
 * that should use that interface in particular ways.
 */

#ifndef VRPSX_DIRECT_TESTS_TEST_ALLOCATOR_H
#define VRPSX_DIRECT_TESTS_TEST_ALLOCATOR_H 1

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "test_dlist.h"
#include "test_where.h"
#include <vrpsx_allocator.h>
#include <vrpsx_stdint.h>


typedef struct test_allocation_struct {
    dlink_t element;
    size_t size; /* payload size NOT including the test_allocation_t */
    vrpsx_uint32_t magic;
    vrpsx_allocator_t *allocator;
} test_allocation_t;

/* Implements the vrpsx_allocator_t interface.
 *
 * Supports multiple allocations but tracks the
 * maximum number of simultaneous allocations.
 */
typedef struct test_allocator_struct {
    vrpsx_allocator_t interface;
    dlink_t allocations;
    size_t current_allocations;
    size_t maximum_allocations;
    int nul_terminated;  /* Cleared when freeing a block whose final
                          * octet was overwritten by a non-NUL value;
                          * is NOT affected by outstanding allocations. */
} test_allocator_t;

extern void test_allocator_construct(
    test_allocator_t *obj);

extern void test_allocator_destroy(
    test_allocator_t *obj);

/* Check that the singular allocation guarantee has been met.
 *
 * Returns 0 if the guarantee is met; otherwise returns -1.
 */
extern int assert_singular_allocation(
    where_t where,          /* The source location to report on failure. */
    test_allocator_t *obj,  /* The allocator to which the guarantee applied. */
    void *block,            /* The memory block allocated by the operation with
                             * the given allocator; NULL if none returned. */
    size_t size);           /* The size of allocation that should have been
                             * made; unused if (block == NULL). */

/* Check that the NUL-terminated allocation guarantee has been met.
 *
 * Returns 0 if the guarantee is met; otherwise returns -1.
 */
extern int assert_nul_terminated_allocation(
    where_t where,          /* The source location to report on failure. */
    test_allocator_t *obj); /* The allocator to which the guarantee applied. */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* VRPSX_DIRECT_TESTS_TEST_ALLOCATOR_H */
